/**
 * This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Created by Tommaso Latini <tommaso@domotz.com> on 14/03/17.
 */


const RETRY_ATTEMPTS = 5;
const RETRY_DELAY = 3000;

module.exports.factory = function (engineRequest, resourceLocator) {
    var myConsole = resourceLocator.log.decorateLogs();
    return {
        create: function (type, reason) {
            if (typeof engineRequest !== 'object' || !engineRequest.hasOwnProperty('sendWithPromise')) {
                myConsole.warn(' Cannot send event [%s,%s] to engine', type, reason);
            }
            var body = {
                type: type,
                reason: reason,
                time: (new Date()).toISOString()
            };
            var otherOptions = {
                maxAttempts: RETRY_ATTEMPTS,
                retryDelay: RETRY_DELAY
            };
            myConsole.info(' Send to engine [%s, %s]', type, reason);
            return engineRequest.sendWithPromise('/event', 'POST', body, null, null, otherOptions);
        }
    };
};
