/** This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Sends a Wake on Lan magic packet to the specified MAC address
 *
 * Created by Iacopo Papalini <iacopo@domotz.com> on 08/07/16.
 */

var wol = require('node-wol');

var wakeOnLan = function (message) {
    var mac = message.payload.mac;
    return {
        execute: function (callback) {

            wol.wake(mac, function (error) {
                "use strict";
                if (error) {
                    console.error(message.correlationId + " - Cannot send magic wake on lan packet to: " + mac);
                    console.error(error.message + '\n' + error.stack);
                } else {
                    console.info(message.correlationId + ' - Command wake on lan executed : ' + mac);
                }
                callback({});
            });

        },
        describe: function () {
            return message.correlationId + " - Wake on lan message: " + mac;
        }
    };
};


module.exports.command = wakeOnLan;