/** This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 **/

var sandboxRun = function (message, resourceLocator) {
    return {
        execute: function (callback) {
            var cid = message.correlationId;
            var myConsole = resourceLocator.log.decorateLogs().decorate("(" + cid + ")");
            myConsole.debug("Executing command echo: %s", JSON.stringify(message.payload));
            resourceLocator.sandboxRunner(
                message.payload,
                cid,
                createFailure(message, resourceLocator, myConsole, callback),
                createSuccess(message, resourceLocator, myConsole, callback)
            );
        },
        describe: function () {
            return message.correlationId + " - Run in Sandbox, device MAC: " + message.payload.device.hw_address;
        }
    };
};

function createSuccess(message, resourceLocator, myConsole, callback) {
    return function success(result) {
        myConsole.info("Success: Dry Run - %s", message.payload.dry_run);
        if (result && result.variables && !message.payload.dry_run) {
            var domotzApi = resourceLocator.engineRequest;
            domotzApi.sendWithPromise(
                '/device/' + message.payload.device.id + '/driver-variable/' + message.payload.driver_id, 'PUT', result.variables
            ).then(function () {
                myConsole.info("Variables sent back to engine");
            });
        }
        return callback(result);
    };
}

function createFailure(_, __, myConsole, callback) {
    return function failure(result) {
        myConsole.warn("Failure: %s", JSON.stringify(result));
        return callback(result);
    };
}

module.exports.command = sandboxRun;
module.exports.createSuccess = createSuccess;