/** This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 **/
/**
 * Created by Tommaso Latini <tommaso@domotz.com> on 31/03/15.
 *
 */

var udpRequest = function (message, resourceLocator) {
    var myConsole = resourceLocator.log.decorateLogs();

    var udpUtils = resourceLocator.udp.utils;
    var cmdUtils = resourceLocator.utils.command;

    var responseData = '';
    var responseError = '';

    var payload = message.payload;
    var host = payload.host;
    var port = payload.port;
    var burst = payload.burst;
    var timeout = payload.timeout;
    var hexMessage = payload.hex_message;
    var useBoundPort = payload.use_bound_port;
    var responseEncoding = payload.response_encoding || "utf-8";

    var socketClosed = false;
    return {
        execute: function (callback) {
            var id;
            var socket = udpUtils.createSocket();
            var buffer = udpUtils.decode(hexMessage);

            if (useBoundPort === true) {
                try {
                    socket.bind(port);
                } catch (error) {
                    myConsole.error('UDP Bind Error: ' + error.message);
                    callback(cmdUtils.getResponse(null, error));
                    return;
                }
            }

            function closeSocket() {
                if (!socketClosed) {
                    socket.close();
                    socketClosed = true;
                }
            }

            socket.on('listening', function () {
                udpUtils.logListening(socket);
            });

            socket.on('message', function (msg, rinfo) {
                udpUtils.logMessage(msg, rinfo);
                responseData += msg.toString(responseEncoding);
                clearTimeout(id);
                closeSocket();
            });

            socket.on('error', function (error) {
                udpUtils.logError(error);
                responseError += error;
                callback(cmdUtils.getResponse(null, error));
                closeSocket();
            });

            socket.on('close', function () {
                udpUtils.logClose('client');
                callback(cmdUtils.getResponse(responseData, responseError));
            });

            id = setTimeout(function () {
                myConsole.debug('TIMEOUT - Closed UDP socket after ' + timeout +
                    ' milliseconds');
                responseError += 'Timeout';
                closeSocket();
            }, timeout);

            udpUtils.send(socket, buffer, host, port, burst);

        },
        describe: function () {
            return message.correlationId + " - udpRequest - " + host +
                ':' + port + ' - ' + hexMessage.length + ' packets';
        }
    };
};

module.exports.command = udpRequest;