/** This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 **/

const PROTOCOL_NAME = 'bonjour';
const BONJOUR_SEMAPHORE = 'BONJOUR_SEMAPHORE';


var bonjourServiceBrowserFactory = require("./bonjour_service_browser");

module.exports.factory = function (resourceLocator) {
    var logger = resourceLocator.log.decorateLogs();

    var mutex = resourceLocator.utils.mutex;
    var bonjourServiceBrowser = bonjourServiceBrowserFactory.factory(null, resourceLocator);

    function lock() {
        return mutex.lock(BONJOUR_SEMAPHORE, 1);
    }

    function unlock() {
        mutex.unlock(BONJOUR_SEMAPHORE, 1);
    }

    function discover (cb, resourceLocator) {
        if (!lock()) {
            logger.warn("WARNING: BONJOUR Discovery service ALREADY STARTED, ignoring call !");
            return;
        }
        try {
            bonjourServiceBrowser.discover(cb, resourceLocator);
        } catch(err) {
            logger.error("Error in Bonjour Discovery !!!!");
            logger.error(err.stack);
        } finally {
            unlock();
        }
    }

    return {
        discover: discover,
        isDiscoveryInProgress: function () {
            return mutex.isLocked(BONJOUR_SEMAPHORE);
        },
        getProtocolName: function () {
            return PROTOCOL_NAME;
        },
        getProtocolNameUpperCase: function () {
            return PROTOCOL_NAME.toUpperCase();
        }
    };

};
