/** This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz UK LLP
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 **/
/**
 *
 * Created by Massimiliano Cuzzoli <massi@domotz.com> on 29/08/16.
 *
 */

const HTTP_RESULT_OK = 200;
const HTTP_REQUEST_ERROR_LEVEL = 'HTTP_REQUEST_ERROR';
const HTTP_RESPONSE_ERROR_LEVEL = 'HTTP_RESPONSE_ERROR';


var deferredRequestFactory = function (resourceLocator, messageBuilder) {
    var q = resourceLocator.q;
    var request = resourceLocator.request;

    return {
        inspect_upnp_resource: function (options) {
            var deferred = q.defer();

            request(options, function (err, response, body) {
                if (err) {
                    deferred.reject(messageBuilder.buildErrorMessage(HTTP_REQUEST_ERROR_LEVEL, null, err));
                } else {
                    if (response.statusCode !== HTTP_RESULT_OK) {
                        deferred.reject(messageBuilder.buildErrorMessage(HTTP_RESPONSE_ERROR_LEVEL,
                            response.statusCode, body));
                    } else {
                        deferred.resolve(body);
                    }
                }
            });
            return deferred.promise;
        }
    };
};


module.exports.factory = deferredRequestFactory;
