/** This file is part of Domotz Agent.
  * Copyright (C) 2016  Domotz Ltd
  *
  * Domotz Agent is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
  * Domotz Agent is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
  **/
/**
 * Created by Tommaso Latini <tommaso@domotz.com> on 14/12/15.
 *
 * @param resourceLocator
 *
 * As all domotz event. It takes as parameter the whole domotz app hash.
 *
 * The handler launches a fast device discovery for each active interfaces.
 */

var counter = 119;
var numberOfFastDiscoveryCyclesInLowThrottle = 120;

var fastDeviceDiscovery = function(resourceLocator) {
    var myConsole = resourceLocator.log.decorateLogs();
    var status = resourceLocator.status;

    counter++;

    if (status.get() === status.RESTRICTED || status.get() === status.UPDATING) {
        myConsole.warn('Pawn in restricted or updating mode. Cannot perform Discovery');
        return;
    }

    var lowThrottleMode = resourceLocator.settingsManager.isLowThrottleMode();

    if (lowThrottleMode && counter !== numberOfFastDiscoveryCyclesInLowThrottle) {
        myConsole.info('Pawn in low throttle mode. Skipping discovery ' + counter + ', starting at ' + numberOfFastDiscoveryCyclesInLowThrottle);
        return;
    } else {
        myConsole.info('performing discovery');
    }

    if (counter === numberOfFastDiscoveryCyclesInLowThrottle) {
        counter = 0;
    }

    resourceLocator.discovery.ip_network.discoveryCoordinator.performDiscovery();
};

module.exports.handler = fastDeviceDiscovery;
