/** This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Created by Andrea Azzara <a.azzara@domotz.com> on 17/04/19.
 *
 * @param resourceLocator
 *
 */


var getExternalServices = function (resourceLocator) {

    var myConsole = resourceLocator.log.decorateLogs();
    var status = resourceLocator.status;
    var engineRequest = resourceLocator.engineRequest;
    var startTime = 120 * resourceLocator.utils.time.SECOND;
    var fingHeartbeat = resourceLocator.utils.fingHeartbeat;
    var host = resourceLocator.discovery.host;

    myConsole.info('getExternalServices handler');

    if (status.get() === status.RESTRICTED || status.get() === status.UPDATING) {
        myConsole.warn('Pawn in restricted or updating mode. Cannot perform MTR');
        return;
    }

    try {
        processExternalServices();
    } catch (e) {
        myConsole.error('Cannot start fing heartbeat process');
    }

    function startFingHeartbeat (hostname, apiKey, macAddress, gatewayMac) {
        var hb = new fingHeartbeat(hostname, apiKey, macAddress, gatewayMac);
        var checkPeriod = 120 * resourceLocator.utils.time.SECOND;

        myConsole.info('Starting Fing heartbeat with params %s %s %s', hostname, macAddress, gatewayMac);

        hb.start();

        setInterval(function () {
            var status = hb.status();
            myConsole.info('Fing heartbeat status' + JSON.stringify(status));
        }, checkPeriod);
    }

    function processExternalServices () {
        host.localNetworkInfoAvailable.then(function () {
            var gatewayMac = host.getDefaultGatewayMAC();
            var macAddress = host.getMAC(host.getInterface());
            setTimeout(function () {

                return engineRequest.sendWithPromise('/external-service', 'GET').then(function (result) {
                    // myConsole.info('external services configuration ' + JSON.stringify(result));
                    if (result && result.fing_outage_detector && result.fing_outage_detector.hostname &&
                        result.fing_outage_detector.api_key) {
                        var hostname = result.fing_outage_detector.hostname;
                        var apiKey = result.fing_outage_detector.api_key;
                        myConsole.info('Fing heartbeat detector configuration received');
                        startFingHeartbeat(hostname, apiKey, macAddress, gatewayMac);
                    } else {
                        myConsole.info('Fing heartbeat disabled for this user');
                    }
                }).catch(function (error) {
                    myConsole.error('Cannot get external services: ' + JSON.stringify(error));
                });
            }, startTime);
        });

    }

};

module.exports.handler = getExternalServices;
