/** This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * Sends an heartbeat signal to the engine, in order to keep 'ONLINE' the agent's status
 * Created by Iacopo Papalini <iacopo@domotz.com> on 22/07/15.
 */

var counter = 50;
var numberOfHeartBeatCyclesInLowThrottle = 60;

var heartBeat = function (resourceLocator) {
    var myConsole = resourceLocator.log.decorateLogs();
    var status = resourceLocator.status;
    var diagnostic = resourceLocator.diagnostic;

    if (status.get() === status.RESTRICTED || status.get() === status.UPDATING) {
        myConsole.warn('Pawn in restricted or updating mode. Won\'t send heartbeat');
        return;
    }

    counter++;

    var lowThrottleMode = resourceLocator.settingsManager.isLowThrottleMode();
    var heartbeatRequest = resourceLocator.heartbeatRequest;

    myConsole.info('Sending heartbeat to engine');

    var sideEffects = 'status-update';
    if (lowThrottleMode && counter !== numberOfHeartBeatCyclesInLowThrottle) {
        sideEffects = 'none';
    }

    if (counter === numberOfHeartBeatCyclesInLowThrottle) {
        counter = 0;
    }
    var discoveryDiagnostic = '&it=' + diagnostic.getDiscoveryIterationCounter() +
        '&ac=' + diagnostic.getActualDiscoveryCounter() +
        '&md=' + diagnostic.getMissedDiscoveryCounter() +
        '&cpu=' + diagnostic.getCPU() +
        '&mem=' + diagnostic.getMem() +
        '&uptime=' + diagnostic.getUptime() +
        '&loadAvg=' + diagnostic.getLoadAvg() +
        '&cpuN=' + diagnostic.getNumberOfCPUs();
    myConsole.info('side effects ' + sideEffects);
    myConsole.info('diagnostics ' + discoveryDiagnostic);

    heartbeatRequest.send('/heartbeat?side-effects=' + sideEffects + discoveryDiagnostic, 'POST', null);
};

module.exports.handler = heartBeat;
