/** This file is part of Domotz Agent.
  * Copyright (C) 2016  Domotz Ltd
  *
  * Domotz Agent is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
  * Domotz Agent is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
  **/
/**
 * Created by Tommaso Latini <tommaso@domotz.com> on 14/12/15.
 */

function getMtrMessage(count, destination) {
    return {
        correlationId: "EVENT",
        payload: {count: count, destination: destination}
    };
}

var mtr = function(resourceLocator) {
    var myConsole = resourceLocator.log.decorateLogs();
    var status = resourceLocator.status;

    if (status.get() === status.RESTRICTED || status.get() === status.UPDATING) {
        myConsole.warn('Pawn in restricted or updating mode. Cannot perform MTR');
        return;
    }

    var executeCommand = resourceLocator.utils.command.execute;
    var join = resourceLocator.path.join;
    var destination = resourceLocator.settingsManager.getMTRRemoteHost();
    var count = resourceLocator.settingsManager.getMTRPacketNumber();

    myConsole.info('Destination: %s. Number of Packets: %s', destination, count);

    var commandMtrPath = join(__dirname, '..', '..', 'commands', 'network', 'mtr.js');

    executeCommand(commandMtrPath, getMtrMessage(count, destination), resourceLocator,
        function (data, err) {
            if (err) {
                myConsole.warn('Error on MTR execution: ' + JSON.stringify(err));
            } else {
                myConsole.verbose('Success --> Raw Data: ' + JSON.stringify(data));
            }
        }
    );

};

module.exports.handler = mtr;
