/** This file is part of Domotz Agent.
 * Copyright (C) 2021  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * This event handler schedules the daily NETBIOS discovery.
 * It is executed after the 4th device fast discovery (~2m since start)
 * in order to not overload the system.
 * After that it is executed for every discovery cycle, considering that
 * the scanner autonomously skips already scanned devices.
 * The goal is to scan every device daily - the assumption is that the agents
 * reboots every day
 */

var counter = 4;

var handler = function (resourceLocator) {
    var myConsole = resourceLocator.log.decorateLogs();

    if (counter > 0) {
        counter -= 1;
        if (counter) {
            myConsole.info('Skipping next %s cycles', counter);
        }
        return;
    }
    if (counter === 0) {
        myConsole.info("Launched");
        resourceLocator.discovery.netbios.discovery.networkScan();
    }
};

module.exports.handler = handler;
