/**
 * This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Created by Tommaso Latini <tommaso@domotz.com> on 20/09/16.
 */

module.exports.factory = function (lodash, childProcess, Queue, len) {

    var cpus = [];

    for (var i = 0; i < len; i++) {
        var execQueue = new Queue(i, childProcess, lodash);
        cpus.push(execQueue);
    }

    return {
        schedule: function (deferred, command, args, options, prio) {
            if (typeof prio !== 'number') {
                prio = 0;
            }
            if (prio < 0 || prio >= len) {
                throw new Error('');
            }

            var cpu = lodash.minBy(cpus, function (e) {
                return e.status();
            });
            if (!cpu) {
                console.error('Cannot schedule process %s, all executors are busy', command);
                deferred.reject('Cannot schedule process, all executors are busy');
                return;
            }
            console.debug('EXECUTOR - Allocating on executor ' + cpu.getIndex());
            cpu.enqueue([deferred, command, args, options]);
        }
    };
};