/** This file is part of Domotz Agent.
 * Copyright (C) 2021  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
/**
 *
 * Created by Fausto Lelli <fausto@domotz.com> on 08/09/21.
 */

module.exports.post = function (resourceLocator, onException, onSuccess) {
    var myConsole = resourceLocator.log.decorateLogs();
    var tokenExpirationTimeout = null;
    var pollingInterval = null;

    function clearPollingInterval () {
        if (pollingInterval !== null) {
            clearInterval(pollingInterval);
            pollingInterval = null;
        }
    }

    return function (req, res) {
        var activationTokenEntity = resourceLocator.entities.activationToken;
        var configurationUpdater = resourceLocator.configurationUpdater;

        myConsole.info('(%s): %s - %s', req.method, req.url, 'getting authentication token from cloud...');

        if (tokenExpirationTimeout !== null) {
            clearTimeout(tokenExpirationTimeout);
            tokenExpirationTimeout = null;
        }
        clearPollingInterval();

        activationTokenEntity.create()
        .then(function (body) {
            var activationToken = body.token;
            var tokenExpirationTime = body.expires_in_minutes * 60 * 1000;

            pollingInterval = setInterval(function () {
                activationTokenEntity.getActivationUrl(activationToken)
                .then(function (response) {
                    var configurationUrl = response.activation_url;
                    myConsole.info('Received configuration url');
                    configurationUpdater.getInitialConfiguration(configurationUrl);
                    clearPollingInterval();
                });
            }, 10000);

            tokenExpirationTimeout = setTimeout(function () {
                clearPollingInterval();
            }, tokenExpirationTime);

            onSuccess(res, body);
        })
        .catch(function (err) {
            onException(res, err);
        });
    };
};
