/** This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 **/
/**
 *
 * Created by Andrea Azzarà <a.azzara@domotz.com> on 10/01/17.
 */
var helpers = require('../helpers');

module.exports.post = function (resourceLocator, onException, onSuccess) {
    /*
     * Creates a new static ip configuration for Domotz Agent
     * Required JSON attributes in request body (type string): ip_address, gateway, primary_dns, secondary_dns
     * */
    return function (req, res) {
        var payload = req.body;

        if (!helpers.checkParameters(payload, ['ip_address', 'netmask', 'primary_dns', 'secondary_dns', 'gateway'], res, onException)) {
            return;
        }

        payload.type = 'static';

        console.info('API - (%s): %s', req.method, req.url);
        var command = require('../../../commands/network/networkConfig').command({payload: payload}, resourceLocator);
        command.execute(helpers.commandToWeb(onException, onSuccess, res));
    };
};

module.exports.delete = function (resourceLocator, onException, onSuccess) {
    /*
     * Sets DHCP configuration for Domotz Agent
     * */
    return function (req, res) {
        var payload = {
            type: 'dhcp'
        };
        console.info('API - (%s): %s', req.method, req.url);
        var command = require('../../../commands/network/networkConfig').command({payload: payload}, resourceLocator);
        command.execute(helpers.commandToWeb(onException, onSuccess, res));
    };
};

module.exports.get = function (resourceLocator, onException, onSuccess) {
    /*
     * Returns 'static' if eth0 is currently configured as static, 'dhcp' otherwise
     * */
    return function (req, res) {
        var payload = {
            type: 'get'
        };
        console.info('API - (%s): %s', req.method, req.url);
        var command = require('../../../commands/network/networkConfig').command({payload: payload}, resourceLocator);
        command.execute(helpers.commandToWeb(onException, onSuccess, res));
    };
};
