/** This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 **/
/**
 *
 * Created by Andrea Azzara <a.azzara@domotz.com> on 01/02/19.
 */

module.exports.post = function (resourceLocator, onException, onSuccess) {
    var configurator = resourceLocator.configurator;
    var domotzAgent = resourceLocator.entities.agent;
    var registerRequest = resourceLocator.registerRequest;
    var serviceReStarter = resourceLocator.utils.proc.gracefulShutdown;
    var geolocation = resourceLocator.networkTools.geolocation;
    var httpOutputCache = resourceLocator.httpOutCache;
    var removeOtherSettingsFiles = configurator.removeOtherSettingsFiles;
    var vendor, displayName, serialNumber, uid;

    return function (req, res) {

        console.info('API - (%s): %s', req.method, req.url);

        var body = req.body;
        try {
            var registrationSecretHeader = 'x-registration-secret';
            var registrationVendor = 'x-domotz-agent-vendor';

            var registrationSecrets = req.headers[registrationSecretHeader].split(',');
            vendor = req.headers[registrationVendor];

            displayName = body.display_name;
            serialNumber = registrationSecrets[0];
            uid = registrationSecrets[1];
        } catch (err) {
            onException(res, {status: 400, message: err.message});
            return;
        }

        geolocation.getLocation(resourceLocator, function (data) {
                configurator.setGeographicalCoordinates(data);
                registerRequest.sendWithPromise('user/vendor/' + vendor, 'GET', null, function (resp) {

                    domotzAgent.createWithVendor(resp.headers.location, vendor, displayName, serialNumber, uid)
                        .then(configurator.setConfigurationUrl)
                        .then(configurator.sendGeographicalCoordinates)
                        .then(configurator.getConfiguration)
                        .then(configurator.writeConfiguration)
                        .then(httpOutputCache.resetCache)
                        .then(removeOtherSettingsFiles)
                        .then(function () {
                            setTimeout(serviceReStarter, 1000);
                        })
                        .then(function () {
                            onSuccess(res);
                        })
                        .catch(function (err) {
                            onException(res, err);
                        });
                }, null, {
                    doNotFollow204: true
                }).catch(function (err) {
                    onException(res, err);
                });
            }
        );
    };
};
