/** This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Created by Tommaso Latini <tommaso@domotz.com> on 23/12/15.
 */

const MODULE = 'STATUS_MANAGER';
const RESTRICTED = 'RESTRICTED';
const UPDATING = 'UPDATING';
const CONFIGURED = 'CONFIGURED';
const PASSIVE = 'PASSIVE';
const ACTION_REQUIRED = 'ACTION_REQUIRED';

var factory = function (configuration) {

    var statuses = [RESTRICTED, UPDATING, CONFIGURED, PASSIVE, ACTION_REQUIRED];
    var status = configuration !== undefined ? CONFIGURED : RESTRICTED;

    function isIn (value, array) {
        return array.indexOf(value) > -1;
    }

    function get () {
        return status;
    }

    function set (s) {
        if (isIn(s, statuses)) {
            console.info('%s - (%s); Set new status of pawn to [%s]!', MODULE, status, s);
            status = s;
        } else {
            console.warn('%s - (%s): Unknown status: %s', MODULE, status, s);
        }
    }

    return {
        get: get,
        set: set,
        isConfigured: function () {
            return status === CONFIGURED;
        },
        isRestricted: function () {
            return status === RESTRICTED;
        },
        isPassive: function () {
            return status === PASSIVE;
        },
        isActionRequired: function () {
            return status === ACTION_REQUIRED;
        },
        RESTRICTED: RESTRICTED,
        CONFIGURED: CONFIGURED,
        UPDATING: UPDATING,
        PASSIVE: PASSIVE,
        ACTION_REQUIRED: ACTION_REQUIRED
    };
};

module.exports.factory = factory;
