/**
 * This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Created by Fausto Lelli <fausto@domotz.com> on 08/06/2016
 *
 */

var _ = require('lodash');

function sortedHashTable(toSort) {
    var sorted = {};
    var keys = [];

    for (var keyToSort in toSort) {
        if (toSort.hasOwnProperty(keyToSort)) {
            keys.push(keyToSort);
        }
    }
    keys.sort();
    for (var i in keys) {
        if (keys.hasOwnProperty(i)) {
            var sortedKey = keys[i];
            sorted[sortedKey] = toSort[sortedKey];
        }
    }
    return sorted;
}

function sortedDictionaryKeys(dictionary) {
    var keys = [];
    for (var key in dictionary) {
        if (dictionary.hasOwnProperty(key)) {
            keys.push(key);
        }
    }
    keys.sort();
    return keys;
}


function mapArrayToDTO(sourceArray, map) {
    var dto = {};
    _.each(map, function (path, source) {
        var elem = dto;
        path = path.split('.');
        while (path.length > 1) {
            var step = path.shift();
            if (elem[step] === undefined) {
                elem[step] = {};
            }
            elem = elem[step];
        }
        elem[path.shift()] = sourceArray[source];
    });
    console.silly('Data Transfer Object: ' + JSON.stringify(dto, null, 2));
    return dto;
}


function patchObject(original, patch) {
    /// Usage original = patch(original, patch);
    "use strict";

    return _.merge(original, patch);
}

module.exports.mapArrayToDTO = mapArrayToDTO;
module.exports.sortedHashTable = sortedHashTable;
module.exports.sortedDictionaryKeys = sortedDictionaryKeys;
module.exports.patchObject = patchObject;
