/**
 * This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Created by Alessio Sanfratello <asanfratello@domotz.com> on 16/06/17.
 */

function getAvailableModules(resourceLocator, callback) {
    var jamDriver = resourceLocator.jamDriver;
    var mlab = resourceLocator.networkTools.mlabSpeedTest;
    var openvpn = resourceLocator.networkTools.openvpn;
    var snmpv3Supported = resourceLocator.snmpv3Supported;
    var platform = resourceLocator.utils.platform;
    var lodash = resourceLocator.lodash;
    var modules = {
        sandbox: {
            version: "1.4",
        },
        ssh_shell_sequence: {
            version: "1.0",
        },
        tftp_server: {
            version: "1.0",
        },
        fast: {
            version: "",
        },
        snmp_status_check: {
            version: "",
        },
        snmp_overlay: {
            version: "0.1"
        },
        nodejs: {
            version: process.version
        }
    };

    if (platform.isWindows()) {
        modules.scp = {
            version: "pscp",
        };
    } else {
        modules.scp = {
            version: "",
        };
    }

    var q = resourceLocator.q;
    var myConsole = resourceLocator.log.decorateLogs();

    if (snmpv3Supported === true) {
        modules.snmpv3 = {
            version: ""
        };
    }

    if (resourceLocator.netPing) {
        modules.netping = {
            version: ""
        };
    }

    if (mlab.isAvailable()) {
        modules.mlab = {
            version: ""
        };
    }

    var openVPNVersionPromise = openvpn.getVersion();
    var promises = [openVPNVersionPromise, platform.getNmapInfo()];

    if (jamDriver.hasOwnProperty('getFingVersion')) {
        var jamDriverPromise = jamDriver.getFingVersion();
        promises.push(jamDriverPromise);
    } else {
        if (jamDriver.hasOwnProperty('luxulApiVersion')) {
            modules.luxul_blocking_api = {
                version: jamDriver.luxulApiVersion().toString()
            };
        }
    }

    q.allSettled(promises).spread(function (openVPNVersion, nmapInfo, fingDriverVersion) {
        var fulfilled = 'fulfilled';
        if (openVPNVersion && openVPNVersion.state === fulfilled) {
            modules.openvpn = {
                version: openVPNVersion.value
            };
        }

        if (fingDriverVersion && fingDriverVersion.state === fulfilled) {
            modules.fing = {
                version: fingDriverVersion.value
            };
        }

        if (nmapInfo && nmapInfo.state === fulfilled) {
            var nmapVersion = lodash.get(nmapInfo.value, 'nmap')
            var npcapVersion = lodash.get(nmapInfo.value, 'Npcap')
            if (nmapVersion) {
                modules.nmap = {
                    version: nmapVersion
                };
            }
            if (npcapVersion) {
                modules.npcap = {
                    version: npcapVersion
                };
            }
        }
        myConsole.info('Available modules: %s', JSON.stringify(modules));

        callback(modules);
    });
}

module.exports = {
    getAvailableModules: getAvailableModules
};
