/**
 * This file is part of Domotz Agent.
 * Copyright (C) 2016  Domotz Ltd
 *
 * Domotz Agent is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Domotz Agent is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Domotz Agent.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Created by Tommaso Latini <tommaso@domotz.com> on 14/09/15.
 */

const MODULE = 'MUTEX';
var locks = {};

function lock(semaphore, id) {
    if (id === undefined) {
        throw new Error("MUTEX - Cannot lock semaphore " + semaphore + " with undefined id");
    }
    var s = semaphore.toUpperCase();
    if (locks[semaphore] === undefined) {
        locks[semaphore] = id;
        console.info('%s - (%s): Lock obtained with id %s', MODULE, s, id);
        return true;
    } else {
        console.info('%s - (%s): Cannot lock semaphore with id %s', MODULE, s, id);
        return false;
    }
}

function unlock(semaphore, id) {
    var s = semaphore.toUpperCase();
    if (id === undefined || locks[semaphore] !== id) {
        throw new Error("MUTEX - (" + s + "): Cannot unlock with id: " + id);
    }
    console.info('%s - (%s): Released lock with id %s', MODULE, s, id);
    locks[semaphore] = undefined;
}

function safeUnlock(semaphore, id) {
    var s = semaphore.toUpperCase();
    if (id === undefined || locks[semaphore] !== id) {
        console.info('%s - (%s): No lock with id %s', MODULE, s, id);
        return;
    }
    console.info('%s - (%s): Released lock with id %s', MODULE, s, id);
    locks[semaphore] = undefined;
}

function isLocked(semaphore) {
    var locked = locks[semaphore] !== undefined;
    console.debug('%s - (%s): %s', MODULE, semaphore.toUpperCase(), locked ? 'locked' : 'unlocked');
    return locked;
}

module.exports.lock = lock;
module.exports.unlock = unlock;
module.exports.isLocked = isLocked;
module.exports.safeUnlock = safeUnlock;
