#!/bin/sh
FOLDER="out/custom-driver"
URL="https://github.com/domotz/custom-driver"
SANDBOX_PATH="../../../src/sandbox"
CURRENT_PAWN_BRANCH=$(git symbolic-ref --short HEAD)
VERSION=$(cat ../src/utils/modules.js | grep sandbox -A1 | grep version | egrep -wo '".*"' | tr -d '"')
echo "Checking out Domotz Custom Driver repository in $FOLDER"
if [ ! -d "$FOLDER" ] ; then
    echo "Folder does not exist, cloning repository"
    git clone $URL $FOLDER
    cd "$FOLDER"
    git checkout rc
else
    echo "Folder exists, pullig latest rc"
    cd "$FOLDER"
    git checkout rc
    git pull
fi

echo "Setting a new sandbox documentation version in sandbox readme"
if [[ "$OSTYPE" == "darwin"* ]]; then
    sed -i '' 's/\<sup\>Documentation Version: .*\<\/sup\>/\<sup\>Documentation Version: '$VERSION'\<\/sup\>/'  ${SANDBOX_PATH}/readme.md
else
    sed -i 's/\<sup\>Documentation Version: .*\<\/sup\>/\<sup\>Documentation Version: '$VERSION'\<\/sup\>/'  ${SANDBOX_PATH}/readme.md
fi


echo "Generating new sandbox documentation"
jsdoc -c ${SANDBOX_PATH}/doc_conf.json -r ${SANDBOX_PATH} ${SANDBOX_PATH}/readme.md -d .

echo "Pushing new sandbox documentation to custom-drivers RC from branch ${CURRENT_PAWN_BRANCH}"
git add .
git commit -m "${CURRENT_PAWN_BRANCH}: Publishing new sandbox documentation version ${VERSION}"
git push

